#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from threading import Lock
import sys

from PyQt5.QtWidgets import QFrame, QHBoxLayout, QVBoxLayout, QLabel, QProgressBar
from PyQt5.QtGui import QColor
from PyQt5.QtCore import Qt as Qtc
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtGui import QPalette

from gnuradio import gr
import pmt


class LabeledLevelGauge(QFrame):
    # Positions: 1 = above, 2=below, 3=left, 4=right
    def __init__(self, lbl='', barColor='blue', backgroundColor='white', fontColor='black',
                 minValue=0, maxValue=100, maxSize=80, position=1, isVertical=True,
                 isFloat=False, scaleFactor=1, showValue=False, parent=None):
        QFrame.__init__(self, parent)
        self.numberControl = LevelGauge(barColor, backgroundColor, minValue, maxValue,
                                        maxSize, isVertical, isFloat, scaleFactor, showValue,
                                        parent)

        if position < 3:
            layout = QVBoxLayout()
        else:
            layout = QHBoxLayout()

        self.lbl = lbl
        self.showvalue = showValue
        self.isFloat = isFloat
        self.isVertical = isVertical
        self.scaleFactor = scaleFactor

        self.lblcontrol = QLabel(lbl, self)
        self.lblcontrol.setAlignment(Qtc.AlignCenter)

        # For whatever reason, the progressbar doesn't show the number in the bar if it's
        # vertical, only if it's horizontal
        if self.showvalue and (isFloat or self.isVertical):
            textstr = self.buildTextStr(minValue / self.scaleFactor)
            self.lblcontrol.setText(textstr)

        if fontColor != 'default':
            self.lblcontrol.setStyleSheet(
                "QLabel { color : " + fontColor + "; }")

        # add top or left
        if len:
            if position == 1 or position == 3:
                layout.addWidget(self.lblcontrol)
        else:
            self.hasLabel = False

        layout.addWidget(self.numberControl)

        # Add bottom or right
        if len:
            if position == 2 or position == 4:
                layout.addWidget(self.lblcontrol)

        layout.setAlignment(Qtc.AlignCenter | Qtc.AlignVCenter)
        self.setLayout(layout)

        self.show()

    def buildTextStr(self, new_value):
        textstr = ""
        if len(self.lbl) > 0:
            textstr = self.lbl + " - "

        if self.isFloat:
            textstr += "%.2f" % (new_value)
        else:
            textstr += str(new_value)

        return textstr

    def valChanged(self, new_value):
        if not self.showvalue:
            return

        if self.isFloat or self.isVertical:
            if self.lbl:
                textstr = self.buildTextStr(new_value)
                self.lblcontrol.setText(textstr)

    def setValue(self, new_value):
        self.valChanged(new_value)

        if int(self.scaleFactor) != 1:
            new_value = int(new_value * self.scaleFactor)

        self.numberControl.setValue(new_value)


class LevelGauge(QProgressBar):
    # Notifies to avoid thread conflicts on paints
    updateInt = pyqtSignal(int)
    updateFloat = pyqtSignal(float)

    def __init__(self, barColor='blue', backgroundColor='white', minValue=0, maxValue=100,
                 maxSize=80, isVertical=True, isFloat=False, scaleFactor=1, showValue=False,
                 parent=None):
        super().__init__(parent)

        self.updateInt.connect(self.onUpdateInt)
        self.updateFloat.connect(self.onUpdateFloat)

        self.lock = Lock()

        self.maxSize = maxSize

        p = super().palette()

        if backgroundColor != 'default':
            p.setColor(QPalette.Base, QColor(backgroundColor))

        if barColor != 'default':
            p.setColor(QPalette.Highlight, QColor(barColor))

        if backgroundColor != 'default' or barColor != 'default':
            super().setPalette(p)

        if (not isFloat) and showValue:
            super().setFormat("%v")  # This shows the number in the bar itself.
            super().setTextVisible(True)
        else:
            super().setTextVisible(False)

        super().setMinimum(minValue)
        super().setMaximum(maxValue)

        if isVertical:
            super().setOrientation(Qtc.Vertical)
        else:
            super().setOrientation(Qtc.Horizontal)

    def onUpdateInt(self, new_value):
        if new_value > super().maximum():
            new_value = super().maximum()
        elif new_value < super().minimum():
            new_value = super().minimum()

        self.lock.acquire()
        super().setValue(new_value)
        self.lock.release()

    def onUpdateFloat(self, new_value):
        if new_value > super().maximum():
            new_value = super().maximum()
        elif new_value < super().minimum():
            new_value = super().minimum()

        self.lock.acquire()
        super().setValue(new_value)
        self.lock.release()

    def setValue(self, new_value):
        if type(new_value) == int:
            self.updateInt.emit(new_value)
        else:
            self.updateFloat.emit(new_value)


class GrLevelGauge(gr.sync_block, LabeledLevelGauge):
    """
    This block creates a level gauge. The value can be set either
    with a variable or an input message.

    NOTE: This control has some quirks due to the fact that
    QProgressBar only accepts integers.  If you want to work with
    floats, you have to use the scaleFactor to map incoming values
    to the specified min/max range.  For instance if the min/max
    are 0-100 but your incoming values are 0.0-1.0, you will need
    to set a scalefactor of 100.
    """

    def __init__(self, lbl='', barColor='blue', backgroundColor='white', fontColor='black',
                 minValue=0, maxValue=100, maxSize=80, isVertical=True, position=1,
                 isFloat=False, scaleFactor=1, showValue=False, parent=None):
        gr.sync_block.__init__(self, name="LevelGauge",
                               in_sig=None, out_sig=None)
        LabeledLevelGauge.__init__(self, lbl, barColor, backgroundColor, fontColor, minValue,
                                   maxValue, maxSize, position, isVertical, isFloat,
                                   scaleFactor, showValue, parent)
        self.lbl = lbl

        if minValue > maxValue:
            gr.log.error("Min value is greater than max value.")
            sys.exit(1)

        self.message_port_register_in(pmt.intern("value"))
        self.set_msg_handler(pmt.intern("value"), self.msgHandler)

    def msgHandler(self, msg):
        try:
            new_val = pmt.to_python(pmt.cdr(msg))

            if type(new_val) == float or type(new_val) == int:
                super().setValue(new_val)
            else:
                gr.log.error(
                    "Value received was not an int or a float: %s" % str(type(new_val)))

        except Exception as e:
            gr.log.error("Error with message conversion: %s" % str(e))

    def setValue(self, new_value):
        super().setValue(new_value)



#!D:\bld\gnuradio_1655844682689\_h_env\python.exe
#
# Copyright 2007,2008,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import numpy

try:
    from pylab import (
        Button,
        connect,
        draw,
        figtext,
        figure,
        get_current_fig_manager,
        plot,
        rcParams,
        show,
    )
except ImportError:
    print(
        "Please install Python Matplotlib (http://matplotlib.sourceforge.net/) and \
           Python TkInter https://wiki.python.org/moin/TkInter to run this script"
    )
    raise SystemExit(1)

from argparse import ArgumentParser


class draw_iq:
    def __init__(self, filename, options):
        self.hfile = open(filename, "r")
        self.block_length = options.block
        self.start = options.start
        self.sample_rate = options.sample_rate

        self.datatype = numpy.complex64
        # number of bytes per sample in file
        self.sizeof_data = self.datatype().nbytes

        self.axis_font_size = 16
        self.label_font_size = 18
        self.title_font_size = 20
        self.text_size = 22

        # Setup PLOT
        self.fig = figure(1, figsize=(16, 9), facecolor="w")
        rcParams["xtick.labelsize"] = self.axis_font_size
        rcParams["ytick.labelsize"] = self.axis_font_size

        self.text_file = figtext(
            0.10, 0.94, ("File: %s" % filename), weight="heavy", size=self.text_size
        )
        self.text_file_pos = figtext(
            0.10, 0.88, "File Position: ", weight="heavy", size=self.text_size
        )
        self.text_block = figtext(
            0.40,
            0.88,
            ("Block Size: %d" % self.block_length),
            weight="heavy",
            size=self.text_size,
        )
        self.text_sr = figtext(
            0.60,
            0.88,
            ("Sample Rate: %.2f" % self.sample_rate),
            weight="heavy",
            size=self.text_size,
        )
        self.make_plots()

        self.button_left_axes = self.fig.add_axes(
            [0.45, 0.01, 0.05, 0.05], frameon=True
        )
        self.button_left = Button(self.button_left_axes, "<")
        self.button_left_callback = self.button_left.on_clicked(
            self.button_left_click)

        self.button_right_axes = self.fig.add_axes(
            [0.50, 0.01, 0.05, 0.05], frameon=True
        )
        self.button_right = Button(self.button_right_axes, ">")
        self.button_right_callback = self.button_right.on_clicked(
            self.button_right_click
        )

        self.xlim = self.sp_iq.get_xlim()

        self.manager = get_current_fig_manager()
        connect("key_press_event", self.click)
        show()

    def get_data(self):
        self.text_file_pos.set_text(
            "File Position: %d" % (self.hfile.tell() // self.sizeof_data)
        )
        try:
            self.iq = numpy.fromfile(
                self.hfile, dtype=self.datatype, count=self.block_length
            )
        except MemoryError:
            print("End of File")
        else:
            self.reals = numpy.array([r.real for r in self.iq])
            self.imags = numpy.array([i.imag for i in self.iq])
            self.time = numpy.array(
                [i * (1 / self.sample_rate) for i in range(len(self.reals))]
            )

    def make_plots(self):
        # if specified on the command-line, set file pointer
        self.hfile.seek(self.sizeof_data * self.start, 1)

        self.get_data()

        # Subplot for real and imaginary parts of signal
        self.sp_iq = self.fig.add_subplot(
            2, 1, 1, position=[0.075, 0.14, 0.85, 0.67])
        self.sp_iq.set_title(
            ("I&Q"), fontsize=self.title_font_size, fontweight="bold")
        self.sp_iq.set_xlabel(
            "Time (s)", fontsize=self.label_font_size, fontweight="bold"
        )
        self.sp_iq.set_ylabel(
            "Amplitude (V)", fontsize=self.label_font_size, fontweight="bold"
        )
        self.plot_iq = plot(self.time, self.reals, "bo-",
                            self.time, self.imags, "ro-")
        self.sp_iq.set_ylim(
            [
                1.5 * min([self.reals.min(), self.imags.min()]),
                1.5 * max([self.reals.max(), self.imags.max()]),
            ]
        )
        self.sp_iq.set_xlim(self.time.min(), self.time.max())
        draw()

    def update_plots(self):
        self.plot_iq[0].set_data([self.time, self.reals])
        self.plot_iq[1].set_data([self.time, self.imags])
        self.sp_iq.set_ylim(
            [
                1.5 * min([self.reals.min(), self.imags.min()]),
                1.5 * max([self.reals.max(), self.imags.max()]),
            ]
        )
        self.sp_iq.set_xlim(self.time.min(), self.time.max())
        draw()

    def click(self, event):
        forward_valid_keys = [" ", "down", "right"]
        backward_valid_keys = ["up", "left"]

        if find(event.key, forward_valid_keys):
            self.step_forward()

        elif find(event.key, backward_valid_keys):
            self.step_backward()

    def button_left_click(self, event):
        self.step_backward()

    def button_right_click(self, event):
        self.step_forward()

    def step_forward(self):
        self.get_data()
        self.update_plots()

    def step_backward(self):
        # Step back in file position
        if self.hfile.tell() >= 2 * self.sizeof_data * self.block_length:
            self.hfile.seek(-2 * self.sizeof_data * self.block_length, 1)
        else:
            self.hfile.seek(-self.hfile.tell(), 1)
        self.get_data()
        self.update_plots()


def find(item_in, list_search):
    try:
        return list_search.index(item_in) is not None
    except ValueError:
        return False


def main():
    description = "Takes a GNU Radio complex binary file and displays the I&Q data versus time. You can set the block size to specify how many points to read in at a time and the start position in the file. By default, the system assumes a sample rate of 1, so in time, each sample is plotted versus the sample number. To set a true time axis, set the sample rate (-R or --sample-rate) to the sample rate used when capturing the samples."

    parser = ArgumentParser(conflict_handler="resolve",
                            description=description)
    parser.add_argument(
        "-B",
        "--block",
        type=int,
        default=1000,
        help="Specify the block size [default=%(default)r]",
    )
    parser.add_argument(
        "-s",
        "--start",
        type=int,
        default=0,
        help="Specify where to start in the file [default=%(default)r]",
    )
    parser.add_argument(
        "-R",
        "--sample-rate",
        type=float,
        default=1.0,
        help="Set the sampler rate of the data [default=%(default)r]",
    )
    parser.add_argument("file", metavar="FILE",
                        help="Input file with complex samples")

    args = parser.parse_args()

    dc = draw_iq(args.file, args)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from PyQt5 import Qt
from PyQt5.QtWidgets import QFrame, QVBoxLayout
from PyQt5.QtCore import Qt as Qtc

from gnuradio import gr
import pmt


class CheckBoxEx(Qt.QCheckBox):
    def __init__(self, lbl, callback=None):
        Qt.QCheckBox.__init__(self)
        self.setText(lbl)
        self.callback = callback

        self.stateChanged.connect(self.onToggleClicked)

    def onToggleClicked(self):
        if self.callback is not None:
            self.callback(super().isChecked())


class MsgCheckBox(gr.sync_block, QFrame):
    """
   This block creates a variable checkbox. Leave the label blank to
   use the variable id as the label. A checkbox selects between
   two values of similar type, but will stay depressed until
   clicked again. The variable will take on one value or the other
   depending on whether the button is pressed or released.

    This control will also produce a state message matching the
    set values.
    """

    def __init__(self, callback, lbl, pressedReleasedDict, initPressed, alignment,
                 valignment, outputmsgname='value'):
        gr.sync_block.__init__(self, name="MsgCheckBox",
                               in_sig=None, out_sig=None)
        QFrame.__init__(self)

        self.outputmsgname = outputmsgname
        self.chkBox = CheckBoxEx(lbl, self.onToggleClicked)

        layout = QVBoxLayout()

        layout.addWidget(self.chkBox)

        if alignment == 1:
            halign = Qtc.AlignCenter
        elif alignment == 2:
            halign = Qtc.AlignLeft
        else:
            halign = Qtc.AlignRight

        if valignment == 1:
            valign = Qtc.AlignVCenter
        elif valignment == 2:
            valign = Qtc.AlignTop
        else:
            valign = Qtc.AlignBottom

        layout.setAlignment(halign | valign)
        self.setLayout(layout)

        self.callback = callback
        self.pressReleasedDict = pressedReleasedDict

        self.message_port_register_out(pmt.intern("state"))

        if initPressed:
            self.chkctl.setChecked(True)

        self.show()

    def onToggleClicked(self, checked):
        if self.chkBox.isChecked():
            self.callback(self.pressReleasedDict['Pressed'])

            if type(self.pressReleasedDict['Pressed']) == bool:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.from_bool(self.pressReleasedDict['Pressed'])))
            elif type(self.pressReleasedDict['Pressed']) == int:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.from_long(self.pressReleasedDict['Pressed'])))
            elif type(self.pressReleasedDict['Pressed']) == float:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.from_double(self.pressReleasedDict['Pressed'])))
            else:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.intern(self.pressReleasedDict['Pressed'])))
        else:
            self.callback(self.pressReleasedDict['Released'])

            if type(self.pressReleasedDict['Released']) == bool:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.from_bool(self.pressReleasedDict['Released'])))
            elif type(self.pressReleasedDict['Released']) == int:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.from_long(self.pressReleasedDict['Released'])))
            elif type(self.pressReleasedDict['Released']) == float:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.from_double(self.pressReleasedDict['Released'])))
            else:
                self.message_port_pub(pmt.intern("state"),
                                      pmt.cons(pmt.intern(self.outputmsgname),
                                               pmt.intern(self.pressReleasedDict['Released'])))
